/*
 * postprint.c - C source for GNU CHESS
 *
 * Revision: 1991-04-15
 *
 * Copyright (C) 1986, 1987, 1988, 1989, 1990 Free Software Foundation, Inc.
 * Copyright (c) 1988, 1989, 1990  John Stanback
 *
 * This file is part of CHESS.
 *
 * CHESS is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY.  No author or distributor accepts responsibility to anyone for
 * the consequences of using it or for whether it serves any particular
 * purpose or works at all, unless he says so in writing.  Refer to the CHESS
 * General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute CHESS, but
 * only under the conditions described in the CHESS General Public License. A
 * copy of this license is supposed to have been given to you along with
 * CHESS so you can know your rights and responsibilities.  It should be in a
 * file named COPYING.  Among other things, the copyright notice and this
 * notice must be preserved on all copies.
 */
#include <stdio.h>
#include "schah.h"
#define HASHFILE "/usr/local/lib/schah.hash"
#ifdef MSDOS
#include <stdlib.h>
#include <string.h>
#include <time.h>
#define RWA_ACC "r+b"
#define WA_ACC "w+b"
#else
#define RWA_ACC "r+"
#define WA_ACC "w+"
#include <sys/param.h>
#include <sys/types.h>
#endif /* MSDOS */
FILE *hashfile;

#define truescore 0x0001
#define lowerbound 0x0002
#define upperbound 0x0004
#define kingcastle 0x0008
#define queencastle 0x0010

struct fileentry
{
  unsigned char bd[32];
  unsigned char f, t, flags, depth, sh, sl;
};
long i, j;
int nr[MAXDEPTH];
struct fileentry n;
int r, c;
char line[128];
char *l;
short int t;
int cc1, cc2;
char mvstr[3][6];
short int board[64];
void
algbr (short int f, short int t, short int flag)

/*
 * Generate move strings in different formats.
 */

{
  int m3p;

  if (f != t)
    {
      /* algebraic notation */
      mvstr[0][0] = Cxx[column (f)];
      mvstr[0][1] = Rxx[row (f)];
      mvstr[0][2] = Cxx[column (t)];
      mvstr[0][3] = Rxx[row (t)];
      mvstr[0][4] = mvstr[3][0] = '\0';
      if (((mvstr[1][0] = Pxx[board[f]]) == 'P') || (flag & promote))
	{
	  if (mvstr[0][0] == mvstr[0][2])
	    {			/* pawn did not eat */
	      mvstr[2][0] = mvstr[1][0] = mvstr[0][2];	/* to column */
	      mvstr[2][1] = mvstr[1][1] = mvstr[0][3];	/* to row */
	      m3p = 2;
	    }
	  else
	    /* pawn ate */
	    {
	      mvstr[2][0] = mvstr[1][0] = mvstr[0][0];	/* from column */
	      mvstr[2][1] = mvstr[1][1] = mvstr[0][2];	/* to column */
	      mvstr[2][2] = mvstr[0][3];
	      m3p = 3;		/* to row */
	    }
	  mvstr[2][m3p] = mvstr[1][2] = '\0';
	  if (flag & promote)
	    {
	      mvstr[0][4] = mvstr[1][2] = mvstr[2][m3p] = Qxx[flag & pmask];
	      mvstr[1][3] = mvstr[2][m3p + 1] = mvstr[0][5] = '\0';
	    }
	}
      else
	/* not a pawn */
	{
	  mvstr[2][0] = mvstr[1][0];
	  mvstr[2][1] = mvstr[0][1];
	  mvstr[2][2] = mvstr[1][1] = mvstr[0][2];	/* to column */
	  mvstr[2][3] = mvstr[1][2] = mvstr[0][3];	/* to row */
	  mvstr[2][4] = mvstr[1][3] = '\0';
	  strcpy (mvstr[3], mvstr[2]);
	  mvstr[3][1] = mvstr[0][0];
	  if (flag & cstlmask)
	    {
	      if (t > f)
		{
		  strcpy (mvstr[1], "o-o");
		  strcpy (mvstr[2], "O-O");
		}
	      else
		{
		  strcpy (mvstr[1], "o-o-o");
		  strcpy (mvstr[2], "O-O-O");
		}
	    }
	}
    }
  else
    mvstr[0][0] = mvstr[1][0] = mvstr[2][0] = mvstr[3][0] = '\0';
}

int
main (int argc, char **argv)
{
  int f = 0;
  char flbuf[10];
  char *fl;

  if ((hashfile = fopen (HASHFILE, RWA_ACC)) == NULL)
    exit (1);
  for (i = 0; i < MAXDEPTH; i++)
    nr[i] = 0;
  fseek (hashfile, 0L, SEEK_END);
  i = ftell (hashfile) / sizeof (struct fileentry);
  fseek (hashfile, 0L, SEEK_SET);
  printf ("/V 11 72 mul def /L 60 def\n");
  for (j = 0; j < i; j++)
    {
      fread (&n, sizeof (struct fileentry), 1, hashfile);
      if (n.depth)
	{
	  nr[0]++;
	  if (nr[0] == 19)
	    {
	      nr[0] = 1;
	      printf ("showpage\n/V 11 72 mul def\n");
	      printf ("/L 60 def\n");
	      f = 0;
	    }
	  /* now process this entry */
	  strcpy (line, "C ('#[");
	  for (r = 0; r < 8; r++)
	    {
	      l = line + 6 + (7 - r) * 9;
	      for (c = 0; c < 4; c++)
		{
		  cc1 = (n.bd[r * 4 + c] >> 4) & 0xf;
		  cc2 = n.bd[r * 4 + c] & 0xf;
		  board[r * 8 + c * 2] = (int) cc1 & 0x7;
		  board[r * 8 + c * 2 + 1] = (int) cc2 & 0x7;
		  if (cc1 & 0x8)
		    *l++ = Qxx[cc1 & 0x7];
		  else
		    *l++ = Pxx[cc1 & 0x7];
		  if (cc2 & 0x8)
		    *l++ = Qxx[cc2 & 0x7];
		  else
		    *l++ = Pxx[cc2 & 0x7];
		}
	      *l++ = ';';
	    }
	  l--;
	  line[79] = '\0';
	  strcat (line, "]') show");
	  algbr (n.f, n.t, 0);
	  t = (n.sh << 8) + n.sl;
	  /* decode flags */
	  fl = flbuf;
	  if (n.flags & kingcastle)
	    *fl++ = 'k';
	  if (n.flags & queencastle)
	    *fl++ = 'q';
	  if (n.flags & truescore)
	    *fl++ = 't';
	  if (n.flags & lowerbound)
	    *fl++ = 'l';
	  if (n.flags & upperbound)
	    *fl++ = 'u';
	  *fl = '\0';
	  printf ("L V moveto\n");
	  printf ("R (%s flags %s depth %d score %d", mvstr[0], flbuf, n.depth, t);
	  printf (") show\n");
	  printf ("L  V 100 sub moveto\n");
	  printf ("%s\n", line);
	  f++;
	  if (f == 3)
	    {
	      printf ("/V V 120 sub def /L 60 def\n");
	      f = 0;
	    }
	  else
	    printf ("/L 160 L add def\n");
	}
    }
  if (nr[0])
    printf ("showpage\n");
}
